Scalar

Struct Scalar 

pub struct Scalar<C: CurveWithScalar> {
    pub(crate) scalar: U448,
    curve: PhantomData<C>,
}
Shared scalar for Ed448 and Decaf448. Use EdwardsScalar and DecafScalar directly.

This is the scalar field size = 4q = 2^446 - 0x8335dc163bb124b65129c96fde933d8d723a70aadc873d6d54a7bb0d We can therefore use 14 saturated 32-bit limbs

Fields

scalar: U448curve: PhantomData<C>

Implementations

Source

impl Scalar<Decaf448>

pub fn from_bytes_mod_order_wide(input: &WideDecafScalarBytes) -> DecafScalar

Construct a Scalar by reducing a 896-bit little-endian integer modulo the group order ℓ.

Source

impl Scalar<Ed448>

pub fn to_bytes_rfc_8032(&self) -> EdwardsScalarBytes

Serialize the scalar into 57 bytes, per RFC 8032. Byte 56 will always be zero.

pub fn from_bytes_mod_order_wide( input: &WideEdwardsScalarBytes, ) -> EdwardsScalar

Construct a Scalar by reducing a 912-bit little-endian integer modulo the group order ℓ.

Source

impl<C: CurveWithScalar> Scalar<C>

pub const ONE: Scalar<C>

The multiplicative identity element

pub const TWO: Scalar<C>

Twice the multiplicative identity element

pub const ZERO: Scalar<C>

The additive identity element

pub(crate) const fn new(scalar: U448) -> Self

pub const fn addition(&self, rhs: &Self) -> Self

Compute self + rhs mod ℓ

pub const fn double(&self) -> Self

Compute self + self mod ℓ

pub const fn subtract(&self, rhs: &Self) -> Self

Compute self - rhs mod ℓ

pub const fn multiply(&self, rhs: &Self) -> Self

Compute self * rhs mod ℓ

pub const fn square(&self) -> Self

Square this scalar

pub fn is_zero(&self) -> Choice

Is this scalar equal to zero?

pub(crate) fn to_radix_16(self) -> [i8; 113]

pub fn bits(&self) -> [bool; 448]

Returns the bits of the scalar in little-endian order.

pub fn to_bytes(&self) -> [u8; 56]

Convert this Scalar to a little-endian byte array.

pub fn invert(&self) -> Self

Invert this scalar

pub const fn exp_vartime(&self, exp: &[u64]) -> Self

Exponentiates self by exp, where exp is a little-endian order integer exponent.

pub fn sqrt(&self) -> CtOption<Self>

Return the square root of this scalar, if it is a quadratic residue.

pub fn div_by_2(&self) -> Self

Halves a Scalar modulo the prime

pub fn from_canonical_bytes( bytes: &Array<u8, <C as CurveWithScalar>::ReprSize>, ) -> CtOption<Self>

Attempt to construct a Scalar from a canonical byte representation.

Return
  • Some(s), where s is the Scalar corresponding to bytes, if bytes is a canonical byte representation;
  • None if bytes is not a canonical byte representation.
pub fn from_bytes_mod_order( input: &Array<u8, <C as CurveWithScalar>::ReprSize>, ) -> Scalar<C>

Construct a Scalar by reducing a 448-bit little-endian integer modulo the group order ℓ

pub fn random<R: Rng + CryptoRng>(rng: &mut R) -> Self

Return a Scalar chosen uniformly at random using a user-provided RNG.

Inputs
  • rng: any RNG which implements the Rng + CryptoRng interface.
Returns

A random scalar within ℤ/lℤ.