pub struct Scalar<C: CurveWithScalar> {
pub(crate) scalar: U448,
curve: PhantomData<C>,
}Expand description
Shared scalar for Ed448 and Decaf448.
Use EdwardsScalar and DecafScalar directly.
This is the scalar field size = 4q = 2^446 - 0x8335dc163bb124b65129c96fde933d8d723a70aadc873d6d54a7bb0d We can therefore use 14 saturated 32-bit limbs
Fields
scalar: U448curve: PhantomData<C>Implementations
Sourceimpl Scalar<Decaf448>
impl Scalar<Decaf448>
Sourcepub fn from_bytes_mod_order_wide(input: &WideDecafScalarBytes) -> DecafScalar
pub fn from_bytes_mod_order_wide(input: &WideDecafScalarBytes) -> DecafScalar
Construct a Scalar by reducing a 896-bit little-endian integer
modulo the group order ℓ.
Sourceimpl Scalar<Ed448>
impl Scalar<Ed448>
Sourcepub fn to_bytes_rfc_8032(&self) -> EdwardsScalarBytes
pub fn to_bytes_rfc_8032(&self) -> EdwardsScalarBytes
Serialize the scalar into 57 bytes, per RFC 8032. Byte 56 will always be zero.
Sourcepub fn from_bytes_mod_order_wide(
input: &WideEdwardsScalarBytes,
) -> EdwardsScalar
pub fn from_bytes_mod_order_wide( input: &WideEdwardsScalarBytes, ) -> EdwardsScalar
Construct a Scalar by reducing a 912-bit little-endian integer
modulo the group order ℓ.
Sourceimpl<C: CurveWithScalar> Scalar<C>
impl<C: CurveWithScalar> Scalar<C>
pub(crate) const fn new(scalar: U448) -> Self
pub(crate) fn to_radix_16(self) -> [i8; 113]
Sourcepub const fn exp_vartime(&self, exp: &[u64]) -> Self
pub const fn exp_vartime(&self, exp: &[u64]) -> Self
Exponentiates self by exp, where exp is a little-endian order integer
exponent.
Sourcepub fn sqrt(&self) -> CtOption<Self>
pub fn sqrt(&self) -> CtOption<Self>
Return the square root of this scalar, if it is a quadratic residue.
Sourcepub fn from_canonical_bytes(
bytes: &Array<u8, <C as CurveWithScalar>::ReprSize>,
) -> CtOption<Self>
pub fn from_canonical_bytes( bytes: &Array<u8, <C as CurveWithScalar>::ReprSize>, ) -> CtOption<Self>
Attempt to construct a Scalar from a canonical byte representation.
Return
Some(s), wheresis theScalarcorresponding tobytes, ifbytesis a canonical byte representation;Noneifbytesis not a canonical byte representation.
Sourcepub fn from_bytes_mod_order(
input: &Array<u8, <C as CurveWithScalar>::ReprSize>,
) -> Scalar<C>
pub fn from_bytes_mod_order( input: &Array<u8, <C as CurveWithScalar>::ReprSize>, ) -> Scalar<C>
Construct a Scalar by reducing a 448-bit little-endian integer modulo the group order ℓ